@@ -59,7 +59,7 @@ static Damage         root_window_damage;
 static XserverRegion  gmag_events_tmp_region;
 #ifdef HAVE_COMPOSITE
 static GQueue        *mag_windows_list;
-static Damage         off_screen_damage;
+static Damage         off_screen_damage = None;
 static Picture        off_screen_picture;
 static XserverRegion  off_screen_region;
 static XserverRegion  tmp_region, new_region, old_region, exp_region;
@@ -68,7 +68,7 @@ static XserverRegion  tmp_region, new_region, old_region, exp_region;
 #endif /* HAVE_XFIXES */
 
 #define EVENTS_DEBUG
-/* #undef  EVENTS_DEBUG */
+#undef  EVENTS_DEBUG
 
 #ifdef  EVENTS_DEBUG
 
@@ -321,6 +321,7 @@ gmag_events_add_window (Window xwin)
 	if (new->attr.class == InputOnly) {
 		new->pic = None;
 		new->damage = None;
+		new->damaged = FALSE;
 		new->damaged_region = None;
 	} else {
 		format = XRenderFindVisualFormat (
@@ -340,31 +341,6 @@ gmag_events_add_window (Window xwin)
 	g_queue_push_tail (mag_windows_list, new);
 }
 
-/*
- * Create the mag_windows_list querying the xserver for the actual
- * windows and adding them to the windows list.
- */
-static void
-gmag_events_create_windows_list ()
-{
-	Window  root_return, parent_return, *children;
-	guint   nchildren;
-	gint    i;
-
-	if (!mag_windows_list)
-		mag_windows_list = g_queue_new ();
-
-	XGrabServer (dpy_conn);
-	XSelectInput (dpy_conn, root_window,
-		      SubstructureNotifyMask);
-	XQueryTree (dpy_conn, root_window,
-		    &root_return, &parent_return, &children, &nchildren);
-	for (i = 0; i < nchildren; i++)
-		gmag_events_add_window (children[i]);
-	XFree (children);
-	XUngrabServer (dpy_conn);
-}
-
 /*
  * Destroy the window resources and remove it from the
  * mag_windows_list.
@@ -389,6 +365,57 @@ gmag_events_remove_window (Window xwin)
 	}
 }
 
+static void
+gmag_events_remove_element (gpointer data, gpointer user_data)
+{
+	GmagWinPtr pgmag_win;
+
+	pgmag_win = (GmagWinPtr) data;
+
+	gmag_events_remove_window (pgmag_win->xwin);
+}
+
+/*
+ * Destroy the mag_windows_list. This is needed when a call to change the
+ * source/target display is made.
+ */
+static void
+gmag_events_destroy_windows_list ()
+{
+	XGrabServer (dpy_conn);
+
+	g_queue_foreach (mag_windows_list, gmag_events_remove_element, None);
+
+	XUngrabServer (dpy_conn);
+}
+
+/*
+ * Create the mag_windows_list querying the xserver for the actual
+ * windows and adding them to the windows list.
+ */
+static void
+gmag_events_create_windows_list ()
+{
+	Window  root_return, parent_return, *children;
+	guint   nchildren;
+	gint    i;
+
+	if (!mag_windows_list)
+		mag_windows_list = g_queue_new ();
+	else
+		gmag_events_destroy_windows_list ();
+
+	XGrabServer (dpy_conn);
+	XSelectInput (dpy_conn, root_window,
+		      SubstructureNotifyMask);
+	XQueryTree (dpy_conn, root_window,
+		    &root_return, &parent_return, &children, &nchildren);
+	for (i = 0; i < nchildren; i++)
+		gmag_events_add_window (children[i]);
+	XFree (children);
+	XUngrabServer (dpy_conn);
+}
+
 /*
  * Add a window damaged region, making a union with the actual damaged region,
  * to the window in mag_windows_list.
@@ -1181,12 +1208,13 @@ gmag_events_client_init (Magnifier *magnifier)
 		new_region = XFixesCreateRegion (dpy_conn, 0, 0);
 		old_region = XFixesCreateRegion (dpy_conn, 0, 0);
 		exp_region = XFixesCreateRegion (dpy_conn, 0, 0);
+		if (off_screen_damage)
+			XDamageDestroy (dpy_conn, off_screen_damage);
 		off_screen_damage = XDamageCreate (
 				dpy_conn, 
 				GDK_DRAWABLE_XID (
 					magnifier->priv->source_drawable),
 				XDamageReportDeltaRectangles);
-
 		format = XRenderFindVisualFormat (
 			dpy_conn,
 			DefaultVisual (dpy_conn, scr));
