@@ -1163,7 +1163,7 @@ magnifier_set_property (BonoboPropertyBag *bag,
 						      full_display_string,
 						      NULL);
 		    magnifier->source_display =
-			gdk_display_open (full_display_string);
+		    	gdk_display_open (full_display_string);
 		    magnifier->source_display_name = g_strdup (full_display_string);
 		    zoom_region_params = magnifier_zoom_regions_save (magnifier);
 		    magnifier->priv->root =
@@ -1220,13 +1220,25 @@ magnifier_set_property (BonoboPropertyBag *bag,
 		    }
 		    magnifier_get_display_rect_bounds (magnifier, &magnifier->source_bounds, FALSE);
 		    magnifier_init_cursor_set (magnifier, magnifier->cursor_set); /* needed to reset pixmaps */
-		    gtk_window_move (GTK_WINDOW (magnifier->priv->w),
-				     magnifier->target_bounds.x1,
-				     magnifier->target_bounds.y1);
+		    if (magnifier->priv->overlay)
+			    gdk_window_move (magnifier->priv->overlay,
+					     magnifier->target_bounds.x1,
+					     magnifier->target_bounds.y1);
+		    else
+			    gtk_window_move (GTK_WINDOW (magnifier->priv->w),
+					     magnifier->target_bounds.x1,
+					     magnifier->target_bounds.y1);
 		    
 		    if ((magnifier->target_bounds.x2 - magnifier->target_bounds.x1 > 0) &&
 			(magnifier->target_bounds.y2 - magnifier->target_bounds.y1) > 0)
 		    {
+			    if (magnifier->priv->overlay)
+				    gdk_window_resize (
+					    magnifier->priv->overlay,
+					    magnifier->target_bounds.x2 -
+					    magnifier->target_bounds.x1,
+					    magnifier->target_bounds.y2 -
+					    magnifier->target_bounds.y1);
 			gtk_window_resize (GTK_WINDOW (magnifier->priv->w),
 				       magnifier->target_bounds.x2 - magnifier->target_bounds.x1,
 				       magnifier->target_bounds.y2 - magnifier->target_bounds.y1);
@@ -1253,10 +1265,20 @@ magnifier_set_property (BonoboPropertyBag *bag,
 	        magnifier->target_bounds = BONOBO_ARG_GET_GENERAL (arg,
 								   TC_GNOME_Magnifier_RectBounds,
 								   GNOME_Magnifier_RectBounds,
-								   NULL);
-		gtk_window_move (GTK_WINDOW (magnifier->priv->w),
-				 magnifier->target_bounds.x1,
-				 magnifier->target_bounds.y1);
+				
+				   NULL);
+		if (magnifier->priv->overlay)
+			gdk_window_move_resize (magnifier->priv->overlay,
+						magnifier->target_bounds.x1,
+						magnifier->target_bounds.y1,
+						magnifier->target_bounds.x2 -
+						magnifier->target_bounds.x1,
+						magnifier->target_bounds.y2 -
+						magnifier->target_bounds.y1);
+		else
+			gtk_window_move (GTK_WINDOW (magnifier->priv->w),
+					 magnifier->target_bounds.x1,
+					 magnifier->target_bounds.y1);
 		
 		gtk_window_resize (GTK_WINDOW (magnifier->priv->w),
 				   magnifier->target_bounds.x2 - magnifier->target_bounds.x1,
@@ -1351,12 +1373,18 @@ impl_magnifier_set_source_display (PortableServer_Servant servant,
 {
 	Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
 	BonoboArg *arg = bonobo_arg_new (BONOBO_ARG_STRING);
-	BONOBO_ARG_SET_STRING (arg, display);
+	gchar *full_display_string;
+
+	full_display_string = g_strdup (display);
+	if (strcmp (full_display_string, "") == 0)
+		full_display_string = (gchar *) g_getenv ("DISPLAY");
 	
-	DBG (fprintf (stderr, "Set source display: \t%s\n", display));
+	BONOBO_ARG_SET_STRING (arg, full_display_string);
 
-	if (strcmp (display, magnifier->source_display_name)) {
+	DBG (fprintf (stderr, "Set source display: \t%s\n",
+		      full_display_string));
 
+	if (strcmp (full_display_string, magnifier->source_display_name)) {
 	    magnifier_set_property (magnifier->property_bag,
 				    arg,
 				    MAGNIFIER_SOURCE_DISPLAY_PROP,
@@ -1366,7 +1394,7 @@ impl_magnifier_set_source_display (PortableServer_Servant servant,
 	else
 	{
 	    DBG (fprintf (stderr, "Attempt to set source to same value as previous: %s\n",
-			  display));
+			  full_display_string));
 	}
 	bonobo_arg_release (arg);
 }
@@ -1378,11 +1406,18 @@ impl_magnifier_set_target_display (PortableServer_Servant servant,
 {
 	Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
 	BonoboArg *arg = bonobo_arg_new (BONOBO_ARG_STRING);
-	BONOBO_ARG_SET_STRING (arg, display);
-	
-	DBG (fprintf (stderr, "Set target display: \t%s\n", display));
+	gchar *full_display_string;
 
-	if (strcmp (display, magnifier->target_display_name)) 
+	full_display_string = g_strdup (display);
+	if (strcmp (full_display_string, "") == 0)
+		full_display_string = (gchar *) g_getenv ("DISPLAY");
+	
+	BONOBO_ARG_SET_STRING (arg, full_display_string);
+	
+	DBG (fprintf (stderr, "Set target display: \t%s\n",
+		      full_display_string));
+
+	if (strcmp (full_display_string, magnifier->target_display_name)) 
 	{
 	    magnifier_set_property (magnifier->property_bag,
 				    arg,
@@ -1393,7 +1428,7 @@ impl_magnifier_set_target_display (PortableServer_Servant servant,
 	else
 	{
 	    DBG (fprintf (stderr, "Attempt to set target to same value as previous: %s\n",
-			  display));
+			  full_display_string));
 	}
 	bonobo_arg_release (arg);
 }
